"""matplatex: export matplotlib figures as pdf and text separately for
use in LaTeX.

Copyright (C) 2024 2025 Johannes Sørby Heines

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

fontsize_map = {
    # Values indicate largest point size for each LaTeX size.
    # These probably need tweaking
    r'\tiny': 4,
    r'\scriptsize': 6,
    r'\footnotesize': 7,
    r'\small': 9,
    r'\normalsize': 11,
    r'\large': 13,
    r'\Large': 15,
    r'\LARGE': 18,
    r'\huge': 25
    }



"""Predefined matplotlib size settings for journals.
"""
from dataclasses import dataclass

_inches_per_mm = 0.03937

@dataclass
class EPJ:
    column_width = 88 * _inches_per_mm
    full_width = 180 * _inches_per_mm
    font_size = 10

@dataclass
class PRC:
    column_width = 3.36
    full_width = 6.75
    font_size = 10

@dataclass
class Beamer:
    full_width = 128 * _inches_per_mm
    widescreen = 160 * _inches_per_mm

    def width(aspectratio: int):
        """Beamer width given aspect ratio. See Beamer documentation 8.3."""
        s = str(aspectratio)
        middle = len(s)//2          # splitting here makes the second slice
        w = s[::-1][middle:][::-1]  # longer in case of odd length. I want
        h = s[::-1][:middle][::-1]  # the first to be longer, so I reverse s.
        return 96 * _inches_per_mm / h * w



"""matplatex: export matplotlib figures as pdf and text separately for
use in LaTeX.

Copyright (C) 2024 2025 Johannes Sørby Heines

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from beartype import beartype
import matplotlib.pyplot as plt

from .tools import write_tex, make_all_transparent, restore_colors
from .latex_input import LaTeXinput

@beartype
def save(
        figure: plt.Figure,
        filename: str,
        *,
        format: str = 'pdf',
        widthcommand: str = r"\figurewidth",
        scale_fontsize: float | bool = 1.0,
        draw_anchors: bool = False,
        externalize: bool = False,
        verbose: int = 1
        ):
    """Save matplotlib Figure with text in a separate tex file.

    Arguments
    ---------
    figure      The matplotlib Figure to save
    filename    The name to use for the files, without extention

    Optional keyword arguments
    --------------------------
    format          The format in which to save the no-text figure.
    widthcommand    The LaTeX length command which will be used to
                    define the width of the figure.
    scale_fontsize  Map the fontsizes in the figure to LaTeX font sizes.
                    If a float is given, the fontsizes are scaled by
                    that value before mapping.
                    If False, no LaTeX font size commands are used.
    draw_anchors    If True, mark the text anchors on the figure.
                    Useful for debugging.
    externalize     Set to True if you want to use tikz externalization.
    verbose: int    0: Print nothing.
                    1: Print save message to stdout. (default)
                    2: Also print runtime info to stderr.
    """
    figure.draw_without_rendering() # Must draw text before it can be extracted.
    layout_engine = figure.get_layout_engine()
    figure.set_layout_engine('none') # Don't change the figure after this.
    output = LaTeXinput(widthcommand=widthcommand, externalize=externalize)
    filename_base = filename.rsplit('/')[-1]
    write_tex(
        output,
        figure,
        graphics=f'{filename_base}_gfx',
        scale_fontsize=scale_fontsize,
        add_anchors=draw_anchors,
        verbose=(verbose==2)
        )
    output.write(f"{filename}.tex")
    color_backup = make_all_transparent(figure)
    figure.savefig(f"{filename}_gfx.{format}", format=format)
    if verbose:
        print(f"Figure written to files {filename}.tex and {filename}_gfx.{format}")
    # restore figure
    restore_colors(figure, color_backup)
    figure.set_layout_engine(layout_engine)


def print_family_tree(mpl_object):
    """Print the family tree of a matplotlib object."""
    stack = [iter(mpl_object.get_children())]
    print(stack)
    indent = ""
    while stack:
        try:
            child = next(stack[-1])
            print(f"{indent}{child}")
            stack.append(iter(child.get_children()))
            indent = indent[:-2]
            indent += "  |- "
        except StopIteration:
            indent = indent[:-5]
            indent += "- "
            stack.pop()



from pathlib import Path
from subprocess import run

import matplotlib.pyplot as plt
import pytest

from matplatex import save

from .latex_test_code import MWE, TIKZEXTERNALIZE

@pytest.fixture
def figure():
    plt.rc('text', usetex=True)
    fig, [ax1, ax2] = plt.subplots(1, 2)
    x1 = [-1, -0.5, -0.3, -0.1, 0.1, 0.3, 0.5, 1]
    y11 = [x**3 for x in x1]  # no numpy to avoid superfluous dependencies
    y12 = [x**2 for x in x1]
    ax1.plot(x1, y11, 'o-', label='$x^3$')
    ax1.plot(x1, y12, 'd-', label='$x^2$')
    ax1.legend()
    ax1.set_xlabel('x', usetex=False)
    ax1.set_ylabel('y')
    ax2.axhline(20)
    ax2.axhspan(0, 17)
    ax2.plot([0, 3, 4, 7], [15, 11, 7, 3], '--')
    return fig

@pytest.fixture(params=[MWE, TIKZEXTERNALIZE])
def latex_source(request, figure, tmp_path):
    latex_path = tmp_path / 'document.tex' # Must match the \pgfrealjobname.
    figure_path = tmp_path / 'figure'
    # Regenerate the files each time so changes are applied.
    externalize = request.param == TIKZEXTERNALIZE
    latex_path.write_text(request.param, encoding='utf-8')
    save(figure, str(figure_path), externalize=externalize, verbose=2)
    return {'dir': tmp_path,
            'file': latex_path.name,
            'ext': externalize
            }

def test_compilation(latex_source):
    # Several asserts here because they are ordered.
    if latex_source['ext']: # requires first compiling figures
        figure_compilation = run(
            ['pdflatex', '--jobname=figure_gfx_xt', latex_source['file']],
            cwd=latex_source['dir']
            )
        assert figure_compilation.returncode == 0
    # compile with pdflatex
    compilation = run(
        ['pdflatex', latex_source['file']],
        cwd=latex_source['dir']
        )
    assert compilation.returncode == 0



MWE = r"""
\documentclass{article}

\usepackage{graphicx}
\usepackage{tikz}

\newlength{\figurewidth}
\newlength{\matplatextmp}

\begin{document}

\setlength{\figurewidth}{\linewidth}
\input{figure.tex}

\end{document}
"""

TIKZEXTERNALIZE = r"""
\documentclass{article}

\usepackage{graphicx}
\usepackage{tikz}
\pgfrealjobname{document}

\newlength{\figurewidth}
\newlength{\matplatextmp}

\begin{document}

\setlength{\figurewidth}{\linewidth}
\input{figure.tex}

\end{document}
"""





from matplotlib import pyplot as plt
from matplatex import save

fig, ax = plt.subplots(figsize=(5, 8))

ax.annotate("This text is rotated", xy=(.3, .9), rotation=40)
ax.annotate("This text is blue", xy=(.4, .8), color='tab:blue')
ax.annotate("This text is Large", xy=(.2, .2), fontsize='x-large')

save(fig, 'test_figure')

plt.show()
